#include <stdio.h>
#include <math.h>
#include <imsls.h>

#define NOBS 200
#define NTHETA 1
#define NB 2
#define NY 1

float fcn(int ntheta, float theta[]);
float *ydata;
int main ()
{
    int  lagma[1];
    float pma[1];
    float *theta; 

    imsls_random_seed_set(123457);
    pma[0] = 0.5;
    lagma[0] = 1;
    ydata = imsls_f_random_arma(200, 0, NULL, 1, pma, 
IMSLS_ACCEPT_REJECT_METHOD,
IMSLS_NONZERO_MALAGS, lagma,
				0);

    theta = imsl_f_min_uncon_multivar(fcn, NTHETA, 0);

    printf("* * * Final Estimate for THETA * * *\n");
    printf("Maximum likelihood estimate, THETA = %f\n", theta[0]);

}

float fcn(int ntheta, float theta[])
{
  int i, n;
  float res, ss, alndet;
  float t[] = {0.0, 1.0, 0.0, 0.0};
  float z[] = {1.0, 0.0};
  float q[NB][NB], r[NY][NY], b[NB], covb[NB][NB], y[NY];
  if (fabs(theta[0]) > 1.0) {
    res = 1.0e10;
  } else {
    q[0][0] = 1.0;
    q[0][1] = -theta[0];
    q[1][0] = -theta[0];
    q[1][1] = theta[0]*theta[0];
    
    r[0][0] = 0.0;
    
    b[0] = 0.0;
    b[1] = 0.0;
    
    covb[0][0] = 1.0 + theta[0]*theta[0];
    covb[0][1] = -theta[0];
    covb[1][0] = -theta[0];
    covb[1][1] = theta[0]*theta[0];
    
    n = 0;
    ss = 0.0;
    alndet = 0.0;
    
    for (i = 0; i<NOBS; i++) {
      y[0] = ydata[i];
      imsls_f_kalman(NB, b, (float*)covb, &n, &ss, &alndet, 
		     IMSLS_UPDATE, NY, y, z, r, 
		     IMSLS_Q, q, 
		     IMSLS_T, t, 
		     0);
    }
    res = n*log(ss/n) + alndet;
  }
  return(res);
}
